from enum import IntEnum


class TimeParam(IntEnum):
    NONE = 0
    T05SEC = 1
    T1SEC = 2
    T2SEC = 3
    T5SEC = 4
    T10SEC = 5
    T15SEC = 6
    T20SEC = 7
    T30SEC = 8
    T45SEC = 9
    T1MIN = 10
    T2MIN = 11
    T5MIN = 12
    T10MIN = 13
    T15MIN = 14
    T20MIN = 15
    T30MIN = 16
    T45MIN = 17
    T60MIN = 18
    T120MIN = 19
